/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 2; tab-width: 2 -*- */
/* gdome-xml-event.c
 *
 * Copyright (C) 1999 Raph Levien <raph@acm.org>
 * Copyright (C) 2000 Mathieu Lacage <mathieu@gnu.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdlib.h>
#include <string.h>
#include "gdome.h"
#include "gdome-events.h"
#include "gdome-util.h"
#include <tree.h>
#include <parser.h>
#include <xmlmemory.h>
#include "gdome-xml-util.h"
#include "gdome-xml-str.h"
#include "gdome-xml-node.h"
#include "gdome-xml-event.h"

const GdomeEventVtab gdome_xml_evnt_vtab = {
  gdome_xml_evnt_ref,
  gdome_xml_evnt_unref,
  gdome_xml_evnt_query_interface,
  gdome_xml_evnt_type,
  gdome_xml_evnt_target,
  gdome_xml_evnt_currentTarget,
  gdome_xml_evnt_eventPhase,
  gdome_xml_evnt_bubbles,
  gdome_xml_evnt_cancelable,
  gdome_xml_evnt_timeStamp,
  gdome_xml_evnt_stopPropagation,
  gdome_xml_evnt_preventDefault,
  gdome_xml_evnt_initEvent
};


void
gdome_xml_evnt_ref (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;

  g_return_if_fail (priv != NULL);
  g_return_if_fail (GDOME_XML_IS_EVNT (priv));
  g_return_if_fail (exc != NULL);

  priv->refcnt++;
}

void
gdome_xml_evnt_unref (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;

  g_return_if_fail (priv != NULL);
  g_return_if_fail (GDOME_XML_IS_EVNT (priv));
  g_return_if_fail (exc != NULL);

  if (--priv->refcnt == 0)
    {
      /* todo: deallocate */
    }
}

void *
gdome_xml_evnt_query_interface (GdomeEvent *self, const char *interface, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;

  g_return_val_if_fail (priv != NULL, NULL);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), NULL);
  g_return_val_if_fail (interface != NULL, NULL);
  g_return_val_if_fail (exc != NULL, NULL);

  if (!strcmp (interface, "Event"))
    {
      priv->refcnt++;
      return self;
    }
  else
    return NULL;
}

GdomeDOMString *
gdome_xml_evnt_type (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_val_if_fail (priv != NULL, NULL);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), NULL);
  g_return_val_if_fail (exc != NULL, NULL);

  ev = priv->ev;
  return gdome_xml_str_mkref (g_strdup (ev->type));
}

GdomeNode *
gdome_xml_evnt_target (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *evnt = NULL;
  GdomeNode *result = NULL;

  g_return_val_if_fail (priv != NULL, NULL);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), NULL);
  g_return_val_if_fail (exc != NULL, NULL);

  evnt = priv->ev;
  result = evnt->target;
  gdome_n_ref (result, exc);
  return result;
}

GdomeNode *
gdome_xml_evnt_currentTarget (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_val_if_fail (priv != NULL, NULL);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), NULL);
  g_return_val_if_fail (exc != NULL, NULL);

  ev = priv->ev;
	
  return gdome_xml_n_mkref (ev->currentTarget);
}

unsigned short
gdome_xml_evnt_eventPhase (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_val_if_fail (priv != NULL, 0);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), 0);
  g_return_val_if_fail (exc != NULL, 0);

  ev = priv->ev;
  return ev->eventPhase;
}

GdomeBoolean
gdome_xml_evnt_bubbles (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_val_if_fail (priv != NULL, 0);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), 0);
  g_return_val_if_fail (exc != NULL, 0);

  ev = priv->ev;
  return ev->bubbles;
}

GdomeBoolean
gdome_xml_evnt_cancelable (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_val_if_fail (priv != NULL, 0);
  g_return_val_if_fail (GDOME_XML_IS_EVNT (priv), 0);
  g_return_val_if_fail (exc != NULL, 0);

  ev = priv->ev;
  return ev->cancelable;
}

GdomeDOMTimeStamp
gdome_xml_evnt_timeStamp (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
	Gdome_xml_EventPriv *ev = priv->ev;

	return ev->timestamp;
}

void
gdome_xml_evnt_stopPropagation (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_if_fail (priv != NULL);
  g_return_if_fail (GDOME_XML_IS_EVNT (priv));
  g_return_if_fail (exc != NULL);

  ev = priv->ev;
  ev->propagation_stopped = TRUE;
}

void
gdome_xml_evnt_preventDefault (GdomeEvent *self, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_if_fail (priv != NULL);
  g_return_if_fail (GDOME_XML_IS_EVNT (priv));
  g_return_if_fail (exc != NULL);

  ev = priv->ev;
  ev->default_prevented = TRUE;
}

void
gdome_xml_evnt_initEvent (GdomeEvent *self, GdomeDOMString *eventTypeArg, GdomeBoolean canBubbleArg, GdomeBoolean cancelableArg, GdomeException *exc)
{
  Gdome_xml_Event *priv = (Gdome_xml_Event *)self;
  Gdome_xml_EventPriv *ev = NULL;

  g_return_if_fail (priv != NULL);
  g_return_if_fail (GDOME_XML_IS_EVNT (priv));
  g_return_if_fail (eventTypeArg != NULL);
  g_return_if_fail (exc != NULL);

  ev = priv->ev;
  /* todo: If you call this on an internally generated event, there
     will be memory problems. Maybe we just need a flag for whether it
     was internally generated or created through DocumentEvent. Maybe
     another flag to check the "before dispatch" phase. */
  if (ev->type != NULL)
    g_free (ev->type);
  ev->type = g_strdup (eventTypeArg->str);
  ev->bubbles = canBubbleArg;
  ev->cancelable = cancelableArg;
}
